import { Component, Input, OnInit } from '@angular/core';
import { animate, style, transition, trigger } from '@angular/animations';
import { Activity, ObservationType } from '@nspop/gm-web-facade-api';
import { ActivitiesService } from '../activities.service';
import { DataCardService } from '../../api/data-card.service';
import { DataCardWrapper } from '../../model-wrappers/dataCard.wrapper';
import ActivityUtil from '../../../utils/activity.util';

@Component({
  selector: 'app-activity-card',
  templateUrl: './activity-card.component.html',
  styleUrls: [ './activity-card.component.scss' ],
  animations: [
    trigger(
      'inOutAnimation',
      [
        transition(
          ':enter',
          [
            style({ opacity: 0 }),
            animate('0.5s ease-out', style({ opacity: 1 }))
          ]
        ),
        transition(
          ':leave',
          [
            style({ opacity: 1 })
          ]
        )
      ]
    )
  ]
})
export class ActivityCardComponent implements OnInit {
  @Input() public activity: Activity | undefined;
  public isExpanded = false;
  public ObservationType = ObservationType; // Exposed here so we can use actual enum in the html template
  public dataCardWrapper: DataCardWrapper | undefined;

  constructor(
    private activitiesService: ActivitiesService,
    public dataCardService: DataCardService
  ) {
  }

  hasMeasurements = () => ActivityUtil.hasMeasurements(this.activity);

  hasRegularMeasurements = () => ActivityUtil.hasRegularMeasurements(this.activity);

  hasUltraSoundMeasurements = () => ActivityUtil.hasUltraSoundMeasurements(this.activity);

  ngOnInit(): void {
    this.dataCardService.selectedDataCard$.subscribe((dataCard) => {
      this.dataCardWrapper = dataCard ? new DataCardWrapper(dataCard) : undefined;
    });
  }

  getFeti(): (string | undefined)[] {
    const fetusNames = this.activity?.observations?.map(observation => observation.fetus);
    if (fetusNames) {
      return fetusNames
        .filter(name => name && name.length > 0)
        .filter((v, i, a) => a.indexOf(v) === i);
    } else {
      return [];
    }
  }

  getActivityStatusColor(status: string): string {
    if (status === 'planned') {
      return '#4228C4';
    } else if (status === 'expected') {
      return '#89C5CC';
    } else if (status === 'finished') {
      return '#646383';
    } else {
      return '#FFF';
    }
  }

  editActivity(): void {
    if (!this.activity) {
      return;
    }
    this.activitiesService.emitOpenActivityModalEvent(this.activity);
  }

  async cancelActivity(): Promise<void> {
    if (!this.activity) {
      return;
    }
    await this.activitiesService.cancelActivity(this.activity);
  }

  addOrEditNote(): void {
    if (!this.activity) {
      return;
    }
    this.activitiesService.emitOpenNoteModalEvent(this.activity, false);
  }

  addOrEditMeasurements(): void {
    if (!this.activity) {
      return;
    }
    this.activitiesService.emitOpenMeasurementsModalEvent(this.activity);
  }

  getActivityDate(): Date | undefined {
    if (this.activity?.conductedTime) {
      return this.activity?.conductedTime;
    }
    return this.activity?.plannedTime?.from;
  }
}
