import { Component, EventEmitter, forwardRef, Input, OnChanges, Output, SimpleChanges } from '@angular/core';
import { RiskAssessment3, SaveUltrasoundObservationsRequest, YesNoEnum } from '@nspop/gm-web-facade-api';
import { ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm } from '@angular/forms';
import { DataCardWrapper } from '../../../model-wrappers/dataCard.wrapper';
import deepEqual from 'deep-equal';
import { copy } from '../../../../utils/copy';
import { ScanningTypeEnum } from '../measurements-modal.component';
import DownsScreeningProbabilityAssessmentEnum = RiskAssessment3.DownsScreeningProbabilityAssessmentEnum;

export const APP_ULTRASOUND_MODAL_SECTION_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => UltrasoundModalSectionComponent),
  multi: true
};

@Component({
  selector: 'app-ultrasound-modal-section',
  templateUrl: './ultrasound-modal-section.component.html',
  styleUrls: [ './ultrasound-modal-section.component.scss' ],
  providers: [ APP_ULTRASOUND_MODAL_SECTION_CONTROL_VALUE_ACCESSOR ],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class UltrasoundModalSectionComponent implements OnChanges, ControlValueAccessor {
  @Input() dataCard: DataCardWrapper | undefined;
  @Input() public readOnlyMode: boolean = false;
  @Input() public isEditing: boolean = false;

  public trimesterScan?: ScanningTypeEnum;
  @Output() public trimesterScanChanged = new EventEmitter<ScanningTypeEnum>();

  public observationDataUltrasound: SaveUltrasoundObservationsRequest | undefined;

  private dirtyAble = false;

  get value(): SaveUltrasoundObservationsRequest | undefined {
    return this.observationDataUltrasound;
  }

  set value(value: SaveUltrasoundObservationsRequest | undefined) {
    if (!deepEqual(value, this.observationDataUltrasound)) {
      this.observationDataUltrasound = value;
      if (this.dirtyAble) {

        this.onChangeCallback(value);
      }
      if (!this.dirtyAble && this.observationDataUltrasound) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
      if (!this.observationDataUltrasound) { // reset dirty when data is reset.
        this.dirtyAble = false;
      }
    }
  }

  get expectedDateOfBirthFromUltrasound(): Date | undefined {
    return this.dataCard?.expectedDateOfBirthFromUltrasound;
  }

  set expectedDateOfBirthFromUltrasound(value: Date | undefined) {
    if (!this.dataCard) {
      return;
    }

    this.dataCard.ultrasoundSection = this.dataCard.ultrasoundSection || {};
    this.dataCard.ultrasoundSection.dirty = true;
    this.dataCard.expectedDateOfBirthFromUltrasound = value;
  }

  get downsProbability(): DownsScreeningProbabilityAssessmentEnum | undefined {
    return this.dataCard?.downsProbability;
  }

  set downsProbability(value: DownsScreeningProbabilityAssessmentEnum | undefined) {
    if (!this.dataCard) {
      return;
    }

    this.dataCard.riskAssessment3Section = this.dataCard.riskAssessment3Section || {};
    this.dataCard.riskAssessment3Section.dirty = true;
    this.dataCard.downsProbability = value;
    this.onChangeCallback(this.value);
  }

  get fetusCount(): number | undefined {
    return this.dataCard?.fetusCount;
  }

  set fetusCount(value: number | undefined) {
    if (!this.dataCard) {
      return;
    }

    this.dataCard.multiplePregnancySection = this.dataCard.multiplePregnancySection || {};
    this.dataCard.multiplePregnancySection.dirty = true;
    this.dataCard.multiplePregnancySection.numberOfFetuses = value;
    this.onChangeCallback(this.value);
  }

  get multiplicityRisk(): YesNoEnum | undefined {
    return this.dataCard?.multiplicityRisk;
  }

  set multiplicityRisk(value: YesNoEnum | undefined) {
    if (!this.dataCard) {
      return;
    }

    this.dataCard.riskAssessment2Section = this.dataCard.riskAssessment2Section || {};
    this.dataCard.riskAssessment2Section.dirty = true;
    this.dataCard.multiplicityRisk = value;
    this.onChangeCallback(this.value);
  }

  get multiplicityRiskComment(): string | undefined {
    return this.dataCard?.multiplicityRiskComment;
  }

  set multiplicityRiskComment(value: string | undefined) {
    if (!this.dataCard) {
      return;
    }

    this.dataCard.riskAssessment2Section = this.dataCard.riskAssessment2Section || {};
    this.dataCard.riskAssessment2Section.dirty = true;
    this.dataCard.multiplicityRiskComment = value;
    this.onChangeCallback(this.value);
  }

  constructor() { }

  public onChangeCallback: (newValue: SaveUltrasoundObservationsRequest | undefined) => any = () => {};

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.dataCard) {
      this.handleFetusCountUpdate(this.dataCard?.fetusCount || 0);
    }
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: SaveUltrasoundObservationsRequest): void {
    this.value = newValue;
  }

  handleFetusCountUpdate(fetusCount: number): void {
    if (this.observationDataUltrasound && this.observationDataUltrasound.fetuses) {
      if (!fetusCount || fetusCount === this.observationDataUltrasound.fetuses.length) {
        return; // If the user is claiming to have 0 feti something is wrong... We'll hold off on deleting stuff
      }
      if (fetusCount < this.observationDataUltrasound.fetuses.length) {
        this.observationDataUltrasound.fetuses = this.observationDataUltrasound.fetuses.slice(0, fetusCount);
      }
      if (fetusCount > this.observationDataUltrasound.fetuses.length) {
        for (let neededFeti = fetusCount - this.observationDataUltrasound.fetuses.length; neededFeti > 0; neededFeti--) {
          this.observationDataUltrasound.fetuses.push(
            { id: `Foster ${this.observationDataUltrasound.fetuses.length + 1}` }
          );
        }
      }
      this.observationDataUltrasound = copy(this.observationDataUltrasound); // Trigger onChange if model has changed
    }
  }

  /**
   * We do not directly set trimester date on the DataCard here,
   * because we don't want to accidentally overwrite a date that is not the final choice.
   *
   * E.g.:
   * 1. select "trimester1" => set date for trimester 1 on DataCard
   * 2. select "trimester2" => set date for trimester 2 on DataCard
   * 3. save. => We now have written both trimester 1 and 2, even though our final choice was 2 - Thus only 2 should've been written
   */
  handleTrimesterScanUpdate(value: ScanningTypeEnum): void {
    if (!this.dataCard) {
      return;
    }

    this.trimesterScanChanged.emit(value);
    this.onChangeCallback(this.value); // Mark form as dirty for updates to this field
  }

  private onTouchedCallback: () => any = () => {};
}
