import { fakeAsync, TestBed } from '@angular/core/testing';
import { UserContext, UserContextService } from './user-context.service';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import { SorItem } from './sor.service';
import { SessionStorageService } from './session-storage.service';
import { PatientContextService } from './patient-context.service';
import { createPatientContextSpyObject } from './patient-context.service.spec';

describe('UserContextService', () => {
  let service: UserContextService;
  let httpTestController: HttpTestingController;
  let patientContextServiceStub: PatientContextService;
  const testBasePath = 'http://localhost:8280/graviditetsmappen';
  const testUserContext: UserContext = {
    commonName: 'Lars Larsen',
    organizationName: 'Test',
    userIdentifier: 123,
    sboLogin: false,
    eligibleRoles: [
      {
        authorizationCode: 'J0184',
        educationCode: '7170',
        educationType: 'Læge'
      },
      {
        authorizationCode: 'J0184',
        educationCode: '7170',
        educationType: 'Jordemoder'
      }
    ]
  };

  let sessionStorageService: SessionStorageService;

  beforeEach(() => {
    patientContextServiceStub = createPatientContextSpyObject();
    sessionStorageService = {
      getUserContext: () => {},
      setUserContext: () => {},
      setUserContextCommonName: () => {},
      setUserContextRole: () => {},
      setUserContextOrganization: () => {},
      deleteUserContext: () => {},
      setUserIdentifier: () => {},

      getPatientContext: () => ({ cpr: '12345'}),
      setPatientContext: () => {},
      setPatientContextCpr: () => {},
      setPatientContextEpisodeOfCare: () => {},
      setPatientContextXBreakTheGlassReason: () => {},
      deletePatientContext: () => {},

      clear: () => {}
    } as any;

    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
      providers: [
        {provide: SessionStorageService, useValue: sessionStorageService},
      ]
    });
    httpTestController = TestBed.inject(HttpTestingController);
    service = TestBed.inject(UserContextService);
  });

  afterEach(() => {
    httpTestController.verify();
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(testUserContext);
  });

  it('should fetch userContext', (done) => {
    service.userContext$.subscribe(
    res => {
      expect(res).toBeDefined();
      done();
    }
    );
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(testUserContext);
    service.ngOnDestroy();
  });

  it('should select role when only one is eligible', (done: () => void) => {
    // copy test context and remove one role
    const singleRoleUser = Object.assign({}, testUserContext);
    singleRoleUser.eligibleRoles.pop();
    const eligibleRole = service.eligibleRoles$;
    expect(eligibleRole).toBeDefined();
    const selectedRole = service.selectedRole$;
    expect(selectedRole).toBeDefined();
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(singleRoleUser);
    done();
  });

  it('should select role', (done) => {
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(testUserContext);
    service.selectedRole$.subscribe(res =>  {
      expect(res).toBeDefined();
      done();
    }
    );
    service.selectRole(testUserContext.eligibleRoles[0]);
  });

  it('should select sor', (done) => {
    httpTestController.expectOne(testBasePath + '/api' + '/usercontext').flush(testUserContext);
    service.selectedOrganizationSor$.subscribe(res =>  {
        expect(res).toBeDefined();
        done();
      }
    );
    const sorTest: SorItem = {name: 'Test', institutionOwnerSorId: 'test', sorId: 'test'};
    service.selectOrganizationSor(sorTest);
  });
  it('should open a patient', fakeAsync(() => {
    httpTestController.expectOne('http://localhost:8280/graviditetsmappen/api/usercontext');
    service.openPatient('12341234-1234');
    httpTestController.expectOne('http://localhost:8280/graviditetsmappen/api/usercontext/patientIdentifier/12341234-1234')
      .flush(testUserContext);
    httpTestController.verify();
  }));

  it('should return true when context is initialized', () => {
    httpTestController.expectOne(testBasePath + '/api/usercontext')
      .flush(testUserContext);
    expect(service.isContextInitialized()).toBeFalse();
    httpTestController.verify();
  });
  it('should return false when neither role bor org is selected', () => {
    httpTestController.expectOne(testBasePath + '/api/usercontext')
      .flush(testUserContext);
    expect(service.isRoleAndOrgSelected()).toBeFalse();
    httpTestController.verify();
  });
  it('should invoke keepAlive', () => {
    service.keepAlive();
    httpTestController.expectOne(testBasePath + '/api/usercontext')
      .flush(testUserContext);
    httpTestController.expectOne(testBasePath + '/api/usercontext/keepAlive')
      .flush(testUserContext);
    httpTestController.verify();
  });

  it('should select an organization', () => {
    httpTestController.expectOne(testBasePath + '/api/usercontext')
      .flush(testUserContext);
    service.selectOrganizationSor({sorId: 'abc', name: 'my org', institutionOwnerSorId: '123'});
    httpTestController.verify();
  });

  it('should set the common name', () => {
    httpTestController.expectOne(testBasePath + '/api/usercontext')
      .flush(testUserContext);
    service.setCommonName('superman');
    httpTestController.verify();
  });
});
