import {
  FetusPresentation,
  ObservationTable,
  ObservationTableFetusesInner,
  ObservationTableMotherRowsInner,
  SaveRegularObservationsRequest,
  SaveRegularObservationsRequestBloodPressure,
  SaveRegularObservationsRequestFetusesInner,
  SaveUltrasoundObservationsRequest,
  SaveUltrasoundObservationsRequestFetusesInner,
  ScanningObservationTable,
  ScanningObservationTableFetusesInner,
  ScanningObservationTableMotherRowsInner
} from '@nspop/gm-web-facade-api';
import FlowEnum = SaveUltrasoundObservationsRequest.FlowEnum;

export interface ActivityClinicalObservationData {
  motherRow?: ObservationTableMotherRowsInner;
  fetusesRows?: ObservationTableFetusesInner[];
}

export interface ActivityScanningObservationData {
  motherRow?: ScanningObservationTableMotherRowsInner;
  fetusesRows?: ScanningObservationTableFetusesInner[];
}

export default class ObservationTableUtil {

  static getClinicalObservationsForActivityId({ activityId, table }: {
    activityId: string | undefined;
    table: ObservationTable | undefined;
  }): ActivityClinicalObservationData {
    const activityClinicalObservationData: ActivityClinicalObservationData = {};

    if (!table || !activityId) {
      return activityClinicalObservationData;
    }

    activityClinicalObservationData.motherRow = table?.mother?.rows?.find(row => row?.activityId === activityId);
    activityClinicalObservationData.fetusesRows = table?.fetuses?.map(fetus => ({
      ...fetus,
      rows: (fetus?.rows || []).filter(fetusRow => fetusRow.activityId === activityId)
    }));

    return activityClinicalObservationData;
  }

  static getScanningObservationsForActivityId({ activityId, table }: {
    activityId: string | undefined;
    table: ScanningObservationTable | undefined;
  }): ActivityScanningObservationData {
    const activityScanningObservationData: ActivityScanningObservationData = {};

    if (!table || !activityId) {
      return activityScanningObservationData;
    }

    activityScanningObservationData.motherRow = table?.mother?.rows?.find(row => row?.activityId === activityId);
    activityScanningObservationData.fetusesRows = table?.fetuses?.map(fetus => ({
      ...fetus,
      rows: (fetus?.rows || []).filter(fetusRow => fetusRow.activityId === activityId)
    }));

    return activityScanningObservationData;
  }

  static hasClinicalObservationsForActivityId(args: {
    activityId: string | undefined;
    table: ObservationTable | undefined;
  }): boolean {
    const { motherRow, fetusesRows } = this.getClinicalObservationsForActivityId(args);
    const hasMotherObservationsForActivityId = !!motherRow;
    const hasFetusObservationsForActivityId = !!fetusesRows?.some(fetus => !!fetus.rows?.length);

    return hasMotherObservationsForActivityId || hasFetusObservationsForActivityId;
  }

  static hasScanningObservationsForActivityId(args: {
    activityId: string | undefined;
    table: ScanningObservationTable | undefined;
  }): boolean {
    const { motherRow, fetusesRows } = this.getScanningObservationsForActivityId(args);
    const hasMotherObservationsForActivityId = !!motherRow;
    const hasFetusObservationsForActivityId = !!fetusesRows?.some(fetus => !!fetus.rows?.length);

    return hasMotherObservationsForActivityId || hasFetusObservationsForActivityId;
  }

  static getObservationDataRegularForActivity({ activityId, table }: {
    activityId: string | undefined;
    table: ObservationTable | undefined;
  }): SaveRegularObservationsRequest {
    const { motherRow, fetusesRows } = this.getClinicalObservationsForActivityId({ activityId, table });
    const fetuses = this.mapToSaveRegularFetus(fetusesRows);

    return {
      activityId,
      documentUniqueId: motherRow?.id || fetusesRows?.[0]?.rows?.[0]?.id, // First fetus, first (and only) observation data row's CDA doc id
      pubismeasurement: motherRow?.pubismeasurement?.value,
      weight: motherRow?.weight?.value,
      bloodPressure: this.mapToBloodPressure(motherRow?.bloodPressure),
      edema: motherRow?.edema,
      urine: motherRow?.urine ? motherRow.urine.split(', ') : undefined,
      fetuses
    };
  }

  static getObservationDataUltrasoundForActivity({ activityId, table }: {
    activityId: string | undefined;
    table: ScanningObservationTable | undefined;
  }): SaveUltrasoundObservationsRequest {
    const { motherRow, fetusesRows } = this.getScanningObservationsForActivityId({ activityId, table });
    const fetuses = this.mapToSaveUltraSoundFetus(fetusesRows);

    return {
      activityId,
      documentUniqueId: motherRow?.id || fetusesRows?.[0]?.rows?.[0]?.id, // First fetus, first (and only) observation data row's CDA doc id
      amnioticFluid: motherRow?.amnioticFluid?.value,
      cervix: motherRow?.cervix?.value,
      dvp: motherRow?.dvp?.value,
      flow: this.mapFlowValue(motherRow?.flow),
      fetuses
    };
  }

  private static mapToSaveRegularFetus(
    clinicalFetuses: ObservationTableFetusesInner[] | undefined
  ): SaveRegularObservationsRequestFetusesInner[] {
    if (!clinicalFetuses?.length) {
      return [];
    }

    return clinicalFetuses?.map(fetus => {
      const fetusId = fetus?.id;
      const row = fetus?.rows?.[0];

      return {
        id: fetusId,
        fetusPresentation: this.mapStringToFetusPresentation(row?.fetusPresentation),
        fetusWeightEstimate: row?.fetusWeightEstimate?.value,
        fetusActivity: row?.fetusActivity
      };
    });
  }

  private static mapToSaveUltraSoundFetus(
    scanningFetuses: ScanningObservationTableFetusesInner[] | undefined
  ): SaveUltrasoundObservationsRequestFetusesInner[] {
    if (!scanningFetuses?.length) {
      return [];
    }

    return scanningFetuses?.map(fetus => {
      const fetusId = fetus?.id;
      const row = fetus?.rows?.[0];

      return {
        id: fetusId,
        fetusPresentation: this.mapStringToFetusPresentation(row?.fetusPresentation),
        ulWeight: row?.ulWeight?.value,
        weightDeviation: row?.weightDeviation?.value
      };
    });
  }

  private static mapToBloodPressure(bloodPressure: string | undefined): SaveRegularObservationsRequestBloodPressure | undefined {
    const bp = bloodPressure?.split('/');
    if (!bp) {
      return;
    }

    return {
      systolic: Number.parseInt(bp[0], 10),
      diastolic: Number.parseInt(bp[1], 10)
    };
  }

  private static mapStringToFetusPresentation(fetusPresentation: string | undefined): FetusPresentation | undefined {
    switch (fetusPresentation?.toUpperCase()) {
      case 'H':
        return FetusPresentation.H;
      case 'TV':
        return FetusPresentation.Tv;
      case 'UK':
        return FetusPresentation.Uk;
    }
  }

  private static mapFlowValue(flow: string | undefined): FlowEnum | undefined {
    switch (flow?.toLowerCase()) {
      case 'normal':
        return FlowEnum.Normal;
      case 'afvigende':
        return FlowEnum.Abnormal;
      case 'der følges op':
        return FlowEnum.Tobecontinued;
    }
  }
}
