import ObservationUtil, { ClinicalObservationTypeEnum, ScanningObservationTypeEnum } from './observation.util';
import { Observation, ObservationValuesInner } from '@nspop/gm-web-facade-api';

function createObservation(...observationValues: Partial<ObservationValuesInner>[]): Observation {
  return { values: observationValues } as Observation;
}

function createBloodPressureObservation({ systolic, diastolic}: {
  systolic?: Partial<ObservationValuesInner> | undefined;
  diastolic?: Partial<ObservationValuesInner> | undefined;
}): Observation {
  return {
    type: 'blood_pressure',
    values: [ systolic ?? {}, diastolic ?? {}]
  } as Observation;
}

describe('ObservationUtil', () => {

  describe('isOfTypeClinicalObservationType', () => {
    it('returns true if the observation type is clinical type', () => {
      expect(ObservationUtil.isOfTypeClinicalObservationType(ClinicalObservationTypeEnum.BLOOD_PRESSURE)).toBe(true);
    });

    it('returns false if the observation type is not a clinical type', () => {
      expect(ObservationUtil.isOfTypeClinicalObservationType(ScanningObservationTypeEnum.CERVIX)).toBe(false);
      expect(ObservationUtil.isOfTypeClinicalObservationType(undefined)).toBe(false);
      expect(ObservationUtil.isOfTypeClinicalObservationType(1 as any)).toBe(false);
      expect(ObservationUtil.isOfTypeClinicalObservationType('hehehe' as any)).toBe(false);
    });
  });

  describe('isOfTypeScanningObservationType', () => {
    it('returns true if the observation type is scanning type', () => {
      expect(ObservationUtil.isOfTypeScanningObservationType(ScanningObservationTypeEnum.CERVIX)).toBe(true);
    });

    it('returns false if the observation type is not a scanning type', () => {
      expect(ObservationUtil.isOfTypeScanningObservationType(ClinicalObservationTypeEnum.BLOOD_PRESSURE)).toBe(false);
      expect(ObservationUtil.isOfTypeScanningObservationType(undefined)).toBe(false);
      expect(ObservationUtil.isOfTypeScanningObservationType(1 as any)).toBe(false);
      expect(ObservationUtil.isOfTypeScanningObservationType('hehehe' as any)).toBe(false);
    });
  });

  describe('formatObservation', () => {
    it('formats a simple observation correctly', () => {
      const simpleWithoutValueOrUnitFormatted = ObservationUtil.formatObservation({ observation: createObservation({}) });
      expect(simpleWithoutValueOrUnitFormatted).toEqual(ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT);

      const simpleWithoutValueFormatted = ObservationUtil.formatObservation({ observation: createObservation({ unit: 'g' }) });
      expect(simpleWithoutValueFormatted).toEqual(`${ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT} g`);

      const simpleWithoutUnitFormatted = ObservationUtil.formatObservation({ observation: createObservation({ value: '100' }) });
      expect(simpleWithoutUnitFormatted).toEqual('100');

      const simpleWithUnitFormatted = ObservationUtil.formatObservation({ observation: createObservation({ value: '999', unit: 'g' }) });
      expect(simpleWithUnitFormatted).toEqual('999 g');

    });

    it('formats an undefined observation correctly', () => {
      const simpleWithoutUnitFormatted = ObservationUtil.formatObservation({ observation: undefined });
      expect(simpleWithoutUnitFormatted).toEqual(ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT);
    });

    it('formats a multi-value observation correctly', () => {
      const multiValueFormatted = ObservationUtil.formatObservation({
        observation: createObservation({}, { unit: 'cm'}, { value: '100' }, { value: '99', unit: 'g'})
      });
      expect(multiValueFormatted).toEqual(`${ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT}, ${ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT} cm, 100, 99 g`);
    });

    it('correctly inserts defined separators while formatting', () => {
      const slashSeparatorFormatted = ObservationUtil.formatObservation({
        observation: createObservation({value: '5'}, { value: '7'}),
        separator: '/'
      });
      expect(slashSeparatorFormatted).toEqual('5/7');

      const dotSeparatorFormatted = ObservationUtil.formatObservation({
        observation: createObservation({value: '127'}, { value: '0'}, { value: '0'}, { value: '1'}),
        separator: '.'
      });
      expect(dotSeparatorFormatted).toEqual('127.0.0.1');
    });

    it('formats blood_pressure observation type correctly', () => {
      const normalFormatted = ObservationUtil.formatObservation({
        observation: createBloodPressureObservation({
          systolic: { value: '100', unit: 'mmHg'},
          diastolic: { value: '140', unit: 'mmHg'}
        })
      });
      expect(normalFormatted).toEqual('100/140 mmHg');

      const systolicOnlyFormatted = ObservationUtil.formatObservation({
        observation: createBloodPressureObservation({
          systolic: { value: '100', unit: 'mmHg'},
          diastolic: { }
        })
      });
      expect(systolicOnlyFormatted).toEqual(`100/${ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT} mmHg`);

      const diastolicOnlyFormatted = ObservationUtil.formatObservation({
        observation: createBloodPressureObservation({
          systolic: { },
          diastolic: { value: '140', unit: 'mmHg'}
        })
      });
      expect(diastolicOnlyFormatted).toEqual(`${ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT}/140 mmHg`);

      const systolicUnitOnlyFormatted = ObservationUtil.formatObservation({
        observation: createBloodPressureObservation({
          systolic: { unit: 'mmHg'},
          diastolic: { }
        })
      });
      expect(systolicUnitOnlyFormatted).toEqual(`${ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT}/${ObservationUtil.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT} mmHg`);

      const BothNoUnitsFormatted = ObservationUtil.formatObservation({
        observation: createBloodPressureObservation({
          systolic: { value: '100'},
          diastolic: { value: '140'}
        })
      });
      expect(BothNoUnitsFormatted).toEqual('100/140');

      const lessThanTwoValuesObservation = {
        type: ClinicalObservationTypeEnum.BLOOD_PRESSURE,
        ...createObservation({ value: '100', unit: 'mmHg'})
      };
      const lessThanToValuesFormatted = ObservationUtil.formatObservation({ observation: lessThanTwoValuesObservation });
      expect(lessThanToValuesFormatted).toEqual(ObservationUtil.DEFAULT_FORMAT_BLOOD_PRESSURE_NOT_TWO_VALUES_TEXT);

      const moreThanTwoValuesObservation = {
        type: ClinicalObservationTypeEnum.BLOOD_PRESSURE,
        ...createObservation({ value: '100', unit: 'mmHg'}, { value: '120', unit: 'mmHg'}, { value: '140', unit: 'mmHg'})
      };
      const moreThanTwoValuesFormatted = ObservationUtil.formatObservation({ observation: moreThanTwoValuesObservation });
      expect(moreThanTwoValuesFormatted).toEqual(ObservationUtil.DEFAULT_FORMAT_BLOOD_PRESSURE_NOT_TWO_VALUES_TEXT);
    });
  });
});
