import { Observation, ObservationType, ObservationValuesInner } from '@nspop/gm-web-facade-api';

export enum ClinicalObservationTypeEnum {
  WEIGHT = 'weight',
  BLOOD_PRESSURE = 'blood_pressure',
  URINE = 'urine',
  EDEMA = 'edema',
  PUBIS_MEASUREMENTS = 'pubis_measurements',
  FETUS_ACTIVITY = 'fetus_activity',
  FETUS_WEIGHT_ESTIMATE = 'fetus_weight_estimate',
  FETUS_PRESENTATION = 'fetus_presentation'
}

export enum ScanningObservationTypeEnum {
  CERVIX = 'cervix',
  DVP = 'dvp',
  GESTRATION_AGE_ULTRASOUND = 'gestation_age_ultrasound',
  AMNIOTIC_FLUID = 'amniotic_fluid',
  FLOW = 'flow',
  FETUS_WEIGHT_ESTIMATE_ULTRASOUND = 'fetus_weight_estimate_ultrasound',
  FETUS_PRESENTATION_ULTRASOUND = 'fetus_presentation_ultrasound',
  FETUS_WEIGHT_DEVIATION = 'weight_deviation'
}

export default class ObservationUtil {

  static DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT = '-';
  static DEFAULT_FORMAT_BLOOD_PRESSURE_NOT_TWO_VALUES_TEXT = '???';
  static DEFAULT_FORMAT_OBSERVATION_SEPARATOR = ', ';

  static isOfTypeClinicalObservationType(type: ObservationType | undefined): boolean {
    if (!type) {
      return false;
    }

    return Object.values(ClinicalObservationTypeEnum).includes(type as ClinicalObservationTypeEnum);
  }

  static isOfTypeScanningObservationType(type: ObservationType | undefined): boolean {
    if (!type) {
      return false;
    }

    return Object.values(ScanningObservationTypeEnum).includes(type as ScanningObservationTypeEnum);
  }

  static formatObservation({
    observation,
    separator = this.DEFAULT_FORMAT_OBSERVATION_SEPARATOR,
    noValueText = this.DEFAULT_FORMAT_OBSERVATION_NO_VALUE_TEXT
  }: {
    observation: Observation | undefined;
    separator?: string;
    noValueText?: string;
  }): string {
    if (!observation) {
      return noValueText;
    }

    if (observation.type === ClinicalObservationTypeEnum.BLOOD_PRESSURE) {
      return this.formatBloodPressureValueUnit({
        observationValues: observation.values,
        notTwoValuesText: this.DEFAULT_FORMAT_BLOOD_PRESSURE_NOT_TWO_VALUES_TEXT,
        noValueText
      });
    }

    return observation.values
      .map(obsValue => this.formatGenericObservationValueUnit({ observationValue: obsValue, noValueText }))
      .join(separator);
  }

  private static formatGenericObservationValueUnit({ observationValue, noValueText }: {
    observationValue: ObservationValuesInner | undefined;
    noValueText: string;
  }): string {
    const { value, unit } = observationValue ?? {};

    return `${value ?? noValueText}${unit ? ' ' + unit : ''}`;
  }

  private static formatBloodPressureValueUnit({ observationValues, noValueText, notTwoValuesText }: {
    observationValues: ObservationValuesInner[];
    noValueText: string;
    notTwoValuesText: string;
  }): string {
    if (observationValues.length !== 2) {
      return notTwoValuesText;
    }

    const [ systolic, diastolic ] = observationValues;

    const systolicValue = systolic?.value;
    const diastolicValue = diastolic?.value;

    const unit = systolic?.unit ?? diastolic?.unit;

    return `${systolicValue ?? noValueText}/${diastolicValue ?? noValueText}${unit ? ' ' + unit : ''}`;
  }
}
